vec3 get_lava_fog(float dist, vec3 color, vec3 fogColor) {
    const vec3 LAVA_FOG_COLOR = vec3(0.9, 0.4, 0.15);
    const vec3 PSNOW_FOG_COLOR = vec3(0.45, 0.67, 1.0);
    
    switch(isEyeInWater) {
        case 1:
            vec3 UnderwaterCol = fogColor*0.5;
            dist = clamp(dist/125, 0, 1);
            dist = 1.0 - exp(-3.0 * dist);
            return mix(color, UnderwaterCol, dist);
        case 2:
            dist = clamp(dist/2, 0, 1);
            return mix(color, LAVA_FOG_COLOR, dist);
        case 3:
            dist = clamp(dist/2, 0, 1);
            return mix(color, PSNOW_FOG_COLOR, dist);
    }
    return color;
}

vec3 get_border_fog(float strength, vec3 color, vec3 SkyColor) {
    strength *= strength;
    #ifndef DIMENSION_NETHER
    strength *= strength;
    strength *= strength;
    #endif
    strength = 1.0 - exp(-3.0 * strength); 
    strength = clamp(strength, 0, 1);
    return mix(color, SkyColor, strength);
}

vec3 get_blindness_fog(float Dist, vec3 Color) {
    if(darknessFactor/4 > blindness) Dist /= 4;
    Dist = clamp(1.0 - exp(-3.0 * Dist/10), 0, 1)*max(darknessFactor, blindness);
    return Color * (1-Dist);
}

vec3 get_atm_fog(float Dist, vec3 Color, vec3 WorldPos) {
    Dist = 1.0 - exp(-3.0 * Dist); 
    Dist *= Dist;
    float Visibility = 0;
    if(worldTime <= 1000) {
        Visibility = cos(PI*(worldTime)/4000.0);
    }
    else if(worldTime >= 12000 && worldTime <= 13000) {
        Visibility = cos(PI*(worldTime-1000)/4000.0);
    }
    else if(worldTime > 13000) {
        Visibility = 1;
    }
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility = max(Visibility, rainStrength*1.5)*is_indoors_smooth();

    vec3 FogColor = apply_saturation(calc_sky_coef()[1], 0.33);
    float HeightFalloff = 1-abs(clamp(WorldPos.y-60, -256, 256)/256);
    HeightFalloff *= HeightFalloff;
    HeightFalloff *= HeightFalloff;
    HeightFalloff *= HeightFalloff;
    float Factor = Dist*HeightFalloff*pow(fbm_fast(WorldPos.xz/32, 2), 0.33)*Visibility;
    vec3 FinalC = mix(Color, FogColor, Factor);
    return mix(FinalC, dither(FinalC, gl_FragCoord.xy), pow(Factor, 0.1));
}