varying vec2 LightmapCoords;
varying vec2 texcoord;
varying vec4 glcolor;
attribute vec4 mc_Entity;
attribute vec2 mc_midTexCoord;
flat varying float material;
varying vec3 ViewPos;
varying vec3 MixedLights;

#include "/lib/settings.glsl"
#include "/lib/tonemap.glsl"

#include "/lib/util.glsl"
#include "/global/sky.glsl"
#include "/lib/blur.glsl"

// [0] is direct, [1] is ambient
vec3[2] get_sun_color() {
    
    const vec3 SUNRISE_SUN = vec3(SUNRISE_RED, SUNRISE_GREEN, SUNRISE_BLUE);
    const vec3 NOON_SUN = vec3(NOON_RED, NOON_GREEN, NOON_BLUE);
    const vec3 SUNSET_SUN = vec3(SUNSET_RED, SUNSET_GREEN, SUNSET_BLUE);

    vec3[2] FinalSun;
    vec3[2] SkyCol = calc_sky_coef();
    
    #ifdef DIMENSION_NETHER
    FinalSun[1] = vec3(0.55, 0.4, 0.3);
    FinalSun[0] = vec3(0);
    return FinalSun;
    #elif defined DIMENSION_END
    FinalSun[1] = vec3(0.4, 0.3, 0.55);
    FinalSun[0] = vec3(0.3, 0.1, 0.0);
    return FinalSun;
    #endif

    if(worldTime > 23500 || worldTime <= 500) {
        float x = sin(PI*(worldTime+500)/2000.0);
        vec3 amb1 = mix(vec3(0.4), mix(SkyCol[1], SkyCol[0], 0.6), 0.30);
        vec3 amb2 = mix(vec3(0.6), mix(SkyCol[1], SkyCol[0], 0.0), 0.3);
        FinalSun[0] = mix(vec3(0), SUNRISE_SUN, x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 1500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        vec3 amb1 = mix(vec3(0.6), mix(SkyCol[1], SkyCol[0], 0.0), 0.3);
        vec3 amb2 = mix(vec3(0.8), mix(SkyCol[1], SkyCol[0], 0.5), 0.4);
        FinalSun[0] = mix(SUNRISE_SUN, NOON_SUN, x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 10500) {
        FinalSun[0] = NOON_SUN;
        FinalSun[1] = mix(vec3(0.8), mix(SkyCol[1], SkyCol[0], 0.5), 0.4);
    }
    else if(worldTime <= 12500) {
        float x = sin(PI*(worldTime-2500.0)/4000.0);
        vec3 amb1 = mix(vec3(0.8), mix(SkyCol[1], SkyCol[0], 0.5), 0.4);
        vec3 amb2 = mix(vec3(0.5), mix(SkyCol[1], SkyCol[0], 0.4), 0.35);
        FinalSun[0] = mix(NOON_SUN, SUNSET_SUN, x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 13500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        vec3 amb1 = mix(vec3(0.5), mix(SkyCol[1], SkyCol[0], 0.4), 0.35);
        vec3 amb2 = mix(vec3(0.4), mix(SkyCol[1], SkyCol[0], 0.6), 0.30);
        FinalSun[0] = mix(SUNSET_SUN, vec3(0), x);
        FinalSun[1] = mix(amb1, amb2, x);
    }
    else if(worldTime <= 23500) {
        FinalSun[0] = vec3(0);
        FinalSun[1] = mix(vec3(0.4), mix(SkyCol[1], SkyCol[0], 0.6), 0.30);
    }
    FinalSun[0] = apply_saturation(FinalSun[0], 1 - rainStrength/2) * (1 - rainStrength*0.33);
    return FinalSun;
}

vec3 obscure_moonlight(vec3 LightColor, float NdotL) {
    float obscure_factor = 0;
    if(worldTime > 22000) obscure_factor = cos(PI*(worldTime+2000.)/4000.0);
    else if(worldTime > 13000 && worldTime <= 14000) obscure_factor = sin(PI*(worldTime-1000.0)/2000.0);
    else if(worldTime > 14000) obscure_factor = 1;

    LightColor = apply_saturation(LightColor, 1 - rainStrength) * (1 - rainStrength*0.25);
    return LightColor*NdotL*obscure_factor;
}

void init_generic() {
    gl_Position = ftransform();
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    LightmapCoords = mat2(gl_TextureMatrix[1]) * gl_MultiTexCoord1.st;
    LightmapCoords = (LightmapCoords * 33.05f / 32.0f) - (1.05f / 32.0f);
    material = mc_Entity.x;
    ViewPos = (gl_ModelViewMatrix * gl_Vertex).xyz;
    vec3 Normal = gl_NormalMatrix * gl_Normal;
    glcolor = gl_Color;
    // Doing this here makes it faster for some reason
    float Dist = length(ViewPos);
    
    float NdotLSun, NdotLMoon;
    #ifndef DIMENSION_NETHER
    if(material == 10001 || material == 10004 || material == 10005 || material == 10006) {
        NdotLSun = NdotLMoon = 0.4;
    }
    else {
        NdotLSun = max(dot(Normal, normalize(sunPosition)), 0);
        NdotLMoon = max(dot(Normal, normalize(moonPosition)), 0);
    }
    #endif

    #ifdef HANDHELD_LIGHTS
    float HandheldLight = max(heldBlockLightValue, heldBlockLightValue2)/15.0;
    LightmapCoords.x = max(LightmapCoords.x, HandheldLight-Dist/15.0);
    #endif

    vec3 TorchColor = vec3(LM_RED, LM_GREEN, LM_BLUE);
    const vec3 MinLight = vec3(MIN_LIGHT_AMOUNT-0.01, MIN_LIGHT_AMOUNT-0.025, MIN_LIGHT_AMOUNT);
     
    #ifndef DIMENSION_OVERWORLD
        LightmapCoords.y = 1;
    #endif

    vec3[2] FinalSun = get_sun_color();
    const vec3 MoonlightColor = vec3(MOON_RED, MOON_GREEN, MOON_BLUE);

    #ifndef DIMENSION_NETHER
    // This cuts off direct sunlight it semi-occulded areas, but only if they are facing up.
    float FakeShadowFactor = max(smoothstep(0.8, 0.95, LightmapCoords.y), (1-max(dot(Normal, normalize(upPosition)), 0)));

    FinalSun[1] += (FinalSun[0] * NdotLSun + obscure_moonlight(MoonlightColor, NdotLMoon)) * FakeShadowFactor;
    #endif 

    #ifdef DIMENSION_OVERWORLD
        float TorchNerfFactor = min(get_luminance(FinalSun[1]), 1);

    #elif defined DIMENSION_END
        float TorchNerfFactor = 0.66;
    #else
        float TorchNerfFactor = 0;
    #endif
    LightmapCoords.x = mix(LightmapCoords.x, mix(LightmapCoords.x, 0, TorchNerfFactor), LightmapCoords.y);
    MixedLights = TorchColor * LightmapCoords.x + FinalSun[1] * LightmapCoords.y;
    MixedLights = mix(MinLight, MixedLights, max(LightmapCoords.x, LightmapCoords.y));
}