#include "/lib/all_the_uniforms.glsl"


#include "/global/lighting.fsh"
#include "/global/fog.glsl"
// Fog, Clouds, Nether & End Sky

varying vec2 texcoord;


/* DRAWBUFFERS:0 */

void main() {
	gl_FragData[0] = texture2D(colortex0, texcoord);

	float Depth = texture2D(depthtex0, texcoord).x;
	vec3 ViewPos = to_view_pos(vec3(texcoord, Depth));
	vec3 PlayerPos = (gbufferModelViewInverse * vec4(ViewPos, 1)).xyz;

	#ifdef DIMENSION_OVERWORLD
	vec3 SkyColor = texture2D(colortex2, texcoord).rgb;
	#elif defined DIMENSION_NETHER
	float LightenFactor = 1.5 - get_luminance(fogColor.xyz); // Makes all biomes roughly equally bright
	LightenFactor *= LightenFactor;
	vec3 SkyColor = apply_saturation(fogColor, 0.7)*LightenFactor;
	#else
	vec3 SkyColor = fogColor*1.5;
	#endif

	float Dist = length(PlayerPos);
	float HorizontalDist = length(PlayerPos.xz);
	if(Depth >= 1) {
		#ifndef DIMENSION_OVERWORLD
		gl_FragData[0].xyz = SkyColor;
		#endif

		#if defined FANCY_CLOUDS && !defined DIMENSION_NETHER
		if(PlayerPos.y > 0) {
			vec3 CloudPos = normalize(PlayerPos); 
			#ifdef DIMENSION_END
			// Adding 5 here makes it look like a black hole
			CloudPos = vec3((CloudPos.xz + 5) * (1 / CloudPos.y), 1).xzy;
			#else
			CloudPos = vec3((CloudPos.xz) * (1 / CloudPos.y), 1).xzy;
			#endif
			gl_FragData[0].xyz += get_clouds(CloudPos);
		}
		#endif

	}
	#ifdef BORDER_FOG
	else {
		#ifdef DIMENSION_OVERWORLD
		gl_FragData[0].xyz = get_atm_fog(Dist/far, gl_FragData[0].xyz, PlayerPos + cameraPosition);
		#endif
		gl_FragData[0].xyz = get_border_fog(HorizontalDist/far, gl_FragData[0].xyz, SkyColor);	
	}
	#endif
	gl_FragData[0].xyz = get_blindness_fog(Dist, gl_FragData[0].xyz);
	gl_FragData[0].xyz = get_lava_fog(Dist, gl_FragData[0].xyz, fogColor);
	
}

